﻿using System.Collections.Generic;
using System.Linq;
using Microsoft.AspNetCore.Mvc;
using ELTE.TodoList.Persistence.Services;
using ELTE.TodoList.Persistence.DTO;
using AutoMapper;
using Microsoft.AspNetCore.Authorization;
using ELTE.TodoList.Persistence;

namespace ELTE.TodoList.WebApi.Controllers
{
    [Route("api/[controller]")]
    [ApiController]
    public class ListsController : ControllerBase
    {
        private readonly ITodoListService _service;
        private readonly IMapper _mapper;

        public ListsController(ITodoListService service, IMapper mapper)
        {
            _service = service;
            _mapper = mapper;
        }

        // GET: api/Lists
        [HttpGet]
        public ActionResult<IEnumerable<ListDto>> GetLists()
        {
            return _service
                .GetLists()
                .Select(list => _mapper.Map<ListDto>(list))
                .ToList();
        }

        // GET: api/Lists/5
        [HttpGet("{id}")]
        public ActionResult<ListDto> GetList(Int32 id)
        {
            try
            {
                var list = _service.GetListById(id);
                return _mapper.Map<ListDto>(list);
            }
            catch (InvalidOperationException)
            {

                return NotFound();
            }
        }

        // PUT: api/Lists/5
        // To protect from overposting attacks, please enable the specific properties you want to bind to, for
        // more details see https://aka.ms/RazorPagesCRUD.
        [Authorize]
        [HttpPut("{id}")]
        public ActionResult PutList(Int32 id, ListDto listDto)
        {
            if (id != listDto.Id)
            {
                return BadRequest();
            }

            if (_service.UpdateList(_mapper.Map<List>(listDto)))
            {
                return Ok();
            }
            else
            {
                return StatusCode(StatusCodes.Status500InternalServerError);
            }
        }

        // POST: api/Lists
        // To protect from overposting attacks, please enable the specific properties you want to bind to, for
        // more details see https://aka.ms/RazorPagesCRUD.
        [Authorize]
        [HttpPost]
        public ActionResult<ListDto> PostList(ListDto listDto)
        {
            var list = _service.CreateList(_mapper.Map<List>(listDto));
            if (list is null)
            {
                return StatusCode(StatusCodes.Status500InternalServerError);
            }
            else
            {
                return CreatedAtAction(nameof(GetList), new { id = list.Id },
                    _mapper.Map<ListDto>(list));
            }
        }

        // DELETE: api/Lists/5
        [Authorize]
        [HttpDelete("{id}")]
        public IActionResult DeleteList(Int32 id)
        {
            if (_service.DeleteList(id))
            {
                return Ok();
            }
            else
            {
                return StatusCode(StatusCodes.Status500InternalServerError);
            }
        }
    }
}
